@@ -3,9 +3,11 @@ Core classes for validation.
 """
 
 import declarative
-import textwrap
-import re
+import gettext
 import os
+import re
+import textwrap
+
 try:
     from pkg_resources import resource_filename
 except ImportError:
@@ -14,7 +16,6 @@ except ImportError:
 __all__ = ['NoDefault', 'Invalid', 'Validator', 'Identity',
            'FancyValidator', 'is_validator']
 
-import gettext
 
 def get_localedir():
     """
@@ -47,30 +48,34 @@ def get_localedir():
 
     return locale_dir
 
-def set_stdtranslation(domain="FormEncode", languages=None, \
+
+def set_stdtranslation(domain="FormEncode", languages=None,
                        localedir = get_localedir()):
 
-    t = gettext.translation(domain=domain, \
-                            languages=languages, \
+    t = gettext.translation(domain=domain,
+                            languages=languages,
                             localedir=localedir, fallback=True)
     global _stdtrans
     _stdtrans = t.ugettext
 
 set_stdtranslation()
 
-def _(s): return s # dummy i18n translation function, nothing is translated here.
-                   # Instead this is actually done in api.Validator.message.
-                   # The surrounding _("string") of the strings is only for extracting
-                   # the strings automatically
-                   # if you run pygettext with this source comment this function out temporarly 
+# Dummy i18n translation function, nothing is translated here.
+# Instead this is actually done in api.Validator.message.
+# The surrounding _('string') of the strings is only for extracting
+# the strings automatically.
+# If you run pygettext with this source comment this function out temporarily.
+_ = lambda s: s
+
+
+class NoDefault(object):
+    """A dummy value used for parameters with no default."""
 
-class NoDefault:
-    pass
 
 def is_validator(obj):
     return (isinstance(obj, Validator) or
-            (isinstance(obj, type) and
-             issubclass(obj, Validator)))
+        (isinstance(obj, type) and issubclass(obj, Validator)))
+
 
 class Invalid(Exception):
 
@@ -101,7 +106,7 @@ class Invalid(Exception):
 
     def __init__(self, msg,
                  value, state, error_list=None, error_dict=None):
-        Exception.__init__(self, msg)
+        Exception.__init__(self, msg, value, state, error_list, error_dict)
         self.msg = msg
         self.value = value
         self.state = state
@@ -119,7 +124,12 @@ class Invalid(Exception):
         return val    
 
     def __unicode__(self):
-        return self.msg
+        if isinstance(self.msg, unicode):
+            return self.msg
+        elif isinstance(self.msg, str):
+            return self.msg.decode('utf8')
+        else:
+            return unicode(self.msg)
 
     def unpack_errors(self, encode_variables=False, dict_char='.',
                       list_char='-'):
@@ -179,24 +189,23 @@ class Validator(declarative.Declarative):
     repeating = False
     compound = False
     gettextargs = {}
-    use_builtins_gettext = True #In case you dont want to use __builtins__._
-                                #altough it may be definied, set this to False
+    use_builtins_gettext = True # In case you don't want to use __builtins__._
+                                # although it may be defined, set this to False
     
     __singletonmethods__ = ('to_python', 'from_python', 'message', 'all_messages',
                             'subvalidators')
 
     def __classinit__(cls, new_attrs):
-        if new_attrs.has_key('messages'):
+        if 'messages' in new_attrs:
             cls._messages = cls._messages.copy()
             cls._messages.update(cls.messages)
             del cls.messages
         cls._initialize_docstring()
 
     def __init__(self, *args, **kw):
-        if kw.has_key('messages'):
+        if 'messages' in kw:
             self._messages = self._messages.copy()
-            self._messages.update(kw['messages'])
-            del kw['messages']
+            self._messages.update(kw.pop('messages'))
         declarative.Declarative.__init__(self, *args, **kw)
 
     def to_python(self, value, state=None):
@@ -206,7 +215,7 @@ class Validator(declarative.Declarative):
         return value
 
     def message(self, msgName, state, **kw):
-        #determine translation function
+        # determine translation function
         try:
             trans = state._
         except AttributeError:
@@ -224,14 +233,15 @@ class Validator(declarative.Declarative):
         if not callable(trans):
             trans = _stdtrans
 
-
+        msg = self._messages[msgName]
+        msg = trans(msg, **self.gettextargs)
         try:
-            return trans(self._messages[msgName], **self.gettextargs) % kw
+            return msg % kw
         except KeyError, e:
             raise KeyError(
                 "Key not found (%s) for %r=%r %% %r (from: %s)"
                 % (e, msgName, self._messages.get(msgName), kw,
-                   ', '.join(self._messages.keys())))        
+                   ', '.join(self._messages.keys())))
 
     def all_messages(self):
         """
@@ -270,7 +280,6 @@ class Validator(declarative.Declarative):
         """
         return []
 
-    #@classmethod
     def _initialize_docstring(cls):
         """
         This changes the class's docstring to include information
@@ -288,11 +297,15 @@ class Validator(declarative.Declarative):
         cls.__doc__ = ''.join(doc)
     _initialize_docstring = classmethod(_initialize_docstring)
 
+
 class _Identity(Validator):
+    
     def __repr__(self):
         return 'validators.Identity'
+    
 Identity = _Identity()
 
+
 class FancyValidator(Validator):
 
     """
